// ======================================================================
// ePost2.cpp - main program of an English Part-of-Speech Tagger (for a
//              parallel corpus)
// 
// 102300: Benjamin Han <benhdj@cs.cmu.edu> Created.
// 110500: Benjamin Han <benhdj@cs.cmu.edu> Added a simple exception for
//         rudimentary file IO error handling.
// 080401: Benjamin Han <benhdj@cs.cmu.edu> Adapted for MSVC++; chaned 
//         license to LGPL.
// 071802: Benjamin Han <benhdj@cs.cmu.edu> Added #include <iterator> for
//         gcc 3.1.
// ======================================================================

//   Copyright (C) 2001 Benjamin Han <benhdj@cs.cmu.edu>
//   The original Transformation-based Part-of-Speech Tagger is written
//   by Eric Brill <brill@cs.jhu.edu> - see the separate copyright notice
//   under post directory
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "post.h"
#include "except.h"

#include <iterator>
#include <iostream>
#include <fstream>
#include <list>

#ifdef __GNUG__
#include <fstream>
#else
#define BUF_SIZE 10240
#endif

#define ERR_SUCCESS                    0
#define ERR_ILLEGAL_OPT                1
#define ERR_OPEN_INPUT_FILE            2
#define ERR_ILLEGAL_CORPUS_FILE_FORMAT 3
#define ERR_POSTERROR                  4

void printHelp ()
{
  cerr<<"* epost2 input-file"<<endl<<endl;

  cerr<<"  input-file: A formatted and tokenized parallel corpus - the 1st sentence of"<<endl;
  cerr<<"              each sentence pair must be in English. WARNING when used the"<<endl;
  cerr<<"              output in Analyzer it's your responsibility to be sure the input"<<endl;
  cerr<<"              file used here are the same file used in running Analyzer!"<<endl<<endl;
  cerr<<"  *IMPORTANT*: DO NOT INCLUDE EMPTY LINES OR LINES WITH ONLY WHITE CHARS!"<<endl<<endl;
}

int main (int argc, char *argv[])
{
#ifdef __GNUG__
  ifstream ifs;
#else
  FILE *fp;
  char buf[BUF_SIZE];
#endif
  list<string> sList;
  string s;
  int i;

  cerr<<"* Brill's English Part-of-Speech Tagger v1.0, adpated by Benjamin Han "<<endl;
  cerr<<"  (for a parallel corpus)"<<endl;

  if (argc!=2)
    {
      printHelp();
      cerr<<"* Illegal command-line syntax."<<endl;
      return ERR_ILLEGAL_OPT;
    }

#ifdef __GNUG__
  // for some mysterious reason VC++ kept spitting "unresolved symbol" on this C++ code
  // that's why the file reading code is changed to the stone-age version
  ifs.open(argv[1]);
  if (!ifs)
    {
      cerr<<"* ERROR: Cannot open "<<argv[1]<<" for reading."<<endl;
      return ERR_OPEN_INPUT_FILE;
    }
#else
  fp=fopen(argv[1],"r");
  if (!fp)
  {
    cerr<<"* ERROR: Cannot open "<<argv[1]<<" for reading."<<endl;
    return ERR_OPEN_INPUT_FILE;
  }
#endif

#ifdef __GNUG__
  for (i=0;getline(ifs,s);)
#else
  for (i=0;fgets(buf,BUF_SIZE,fp);)
#endif
    {
#ifndef __GNUG__
      s=buf;
	  if (s[s.length()-1]='\n') s.erase(s.length()-1);
#endif
      // only put the first sentence of a sentence pair into sList
      if (s.find_first_not_of(" \t")==string::npos) i=0;
      else
	    switch (i)
		{
		  case 0:
	      i=1;
	      sList.push_back(s);
	      break;
	    
	    case 1:
	      i=2;
	      break;

	    case 2:
	      cerr<<"* ERROR: More then 2 consecutive sentences."<<endl;
	      return ERR_ILLEGAL_CORPUS_FILE_FORMAT;
	    
	    default:
	      break;
	    }
    }

#ifdef __GNUG__
  ifs.close();
#else
  fclose(fp);
#endif

  try
    {
      POST post;
      
      post.run(sList);
      
      copy(sList.begin(),sList.end(),ostream_iterator<string>(cout,"\n"));
    }

  catch (Exception &e)
    {
      cerr<<"* ERROR: "<<e<<endl;
      return ERR_POSTERROR;
    }
      
  return ERR_SUCCESS;
}
