// ======================================================================
// except.h - simple exception handling
// 
// 101900: Benjamin Han <benhdj@cs.cmu.edu> Created.
// 080401: Benjamin Han <benhdj@cs.cmu.edu> Adapted for MSVC++; chaned 
//         license to LGPL.
// ======================================================================

//   Copyright (C) 2001 Benjamin Han <benhdj@cs.cmu.edu>
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef EXCEPT_H
#define EXCEPT_H

#define EX_PLACEHOLDER "%s"

// ===========================================================================
// Exception class
// ===========================================================================

#include <string>
#include <iostream>

using namespace std;

class Exception
{
  string msg;

  friend ostream &operator<< (ostream &s, const Exception &tex);

public:

  // When print a Exception the EX_PLACEHOLDER in msg will be
  // replaced by arg.
  Exception (const string &msg, const string &arg)
  {
    int pos;
    Exception::msg=msg;

    if ((pos=Exception::msg.find(EX_PLACEHOLDER))<string::npos)
      {
	Exception::msg.erase(pos,strlen(EX_PLACEHOLDER));
	Exception::msg.insert(pos,arg);
      }
  }
};

inline ostream &operator<< (ostream &s, const Exception &tex)
{
  return s<<tex.msg;
}

// ===========================================================================
// Macro DEFINE_SIMPLE_EXCEPTION: Quickly define a specialized
// exception with customized error message.
// ===========================================================================

#define DEFINE_SIMPLE_EXCEPTION(exClassName,exMsg)\
class exClassName: public Exception\
{\
public:\
exClassName (const string &arg):Exception(exMsg,arg) {}\
}

#endif
