// ======================================================================
// post.h - an English Part-of-Speech Tagger based on Eric Brill's code
// 
// 102300: Benjamin Han <benhdj@cs.cmu.edu> Created.
// 110500: Benjamin Han <benhdj@cs.cmu.edu> Added a simple exception for
//         rudimentary file IO error handling.
// 080401: Benjamin Han <benhdj@cs.cmu.edu> Adapted for MSVC++; chaned 
//         license to LGPL.
// ======================================================================

//   Copyright (C) 2001 Benjamin Han <benhdj@cs.cmu.edu>
//   The original Transformation-based Part-of-Speech Tagger is written
//   by Eric Brill <brill@cs.jhu.edu> - see the separate copyright notice
//   under post directory
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef POST_H
#define POST_H

#include <list>
#include <string>

extern "C"
{
#include "lex.h"
#include "darray.h"
#include "useful.h"
#include "memory.h"
#include "tagger.h"
}

#include "except.h"

using namespace std;

DEFINE_SIMPLE_EXCEPTION(POSTFileError,"File \"%s\" for POST cannot be opened.");

// ===========================================================================
// Wrapper for Brill's part-of-speech parser
// ===========================================================================
class POST
{
  // statr-state-tagger (final-state-tagger also uses lexicon_hash)
  Registry lexicon_hash, good_right_hash, good_left_hash;

  // final-state-tagger
  Registry seenTagging_hash;

  Darray lRuleArray,cRuleArray,bigramArray;
  
public:

  POST ();  // Initializes the hash registries and rule array
  ~POST ();

  // 'sList' initially has an array of untagged sentences; after run() it
  // contains an array of tags corresponding to each original sentence
  // WARNING: DO NOT INCLUDE ANY EMPTY STRING IN sList - CRASH-n-BURN!
  void run (list<string> &sList);
};

#endif
