/* Copyright @ 1993 MIT and University of Pennsylvania */
/* Written by Eric Brill */
/*THIS SOFTWARE IS PROVIDED "AS IS", AND M.I.T. MAKES NO REPRESENTATIONS 
OR WARRANTIES, EXPRESS OR IMPLIED.  By way of example, but not 
limitation, M.I.T. MAKES NO REPRESENTATIONS OR WARRANTIES OF 
MERCHANTABILITY OR FITNESS FOR ANY PARTICULAR PURPOSE OR THAT THE USE OF 
THE LICENSED SOFTWARE OR DOCUMENTATION WILL NOT INFRINGE ANY THIRD PARTY 
PATENTS, COPYRIGHTS, TRADEMARKS OR OTHER RIGHTS.   */


#include <string.h>
#include <stdlib.h>
#include <sys/types.h>

#ifdef __GNUG__
#include <unistd.h>
#endif

#include "darray.h"
#include "useful.h"
#include "tagger.h"
#define MAXTAGLEN 256  /* max char length of pos tags */
#define MAXWORDLEN 256 /* max char length of words */

void Tagger (FILE *lexicon, FILE *bigrams, FILE *lRuleFile, FILE *cRuleFile,
	     Registry *lexicon_hash, Registry *good_right_hash,
	     Registry *good_left_hash, Registry *seenTagging,
	     Darray *bigramArray, Darray *lRuleArray, Darray *cRuleArray)
{
  char line[MAXLINELEN];
  char space[500];
  char COMMAND[1024];
  char word[MAXWORDLEN],tag[MAXTAGLEN];
  char bigram1[MAXWORDLEN],bigram2[MAXWORDLEN];
  char **perl_split_ptr,**perl_split_ptr2,*atempstr,**temp_perl_split_ptr;
  char *tempruleptr;
  char bigram_space[MAXWORDLEN*2];
  int numLexiconEntries;

  /* Benjamin Han 100400: time for creativity! */
  *lexicon_hash = Registry_create(Registry_strcmp,Registry_strhash);
  *good_right_hash = Registry_create(Registry_strcmp,Registry_strhash);
  *good_left_hash = Registry_create(Registry_strcmp,Registry_strhash);
  *seenTagging = Registry_create(Registry_strcmp,Registry_strhash);
  *lRuleArray = Darray_create();
  *cRuleArray = Darray_create();
  *bigramArray = Darray_create();

  /* lexicon hash stores the most likely tag for all known words.
     we can have a separate wordlist and lexicon file because unsupervised
     learning    can add to wordlist, while not adding to lexicon.  For
     example, if a big    untagged corpus is about to be tagged, the wordlist
     can be extended to    include words in that corpus, while the lexicon
     remains static.    Lexicon is file of form: 
     word t1 t2 ... tn 
     where t1 is the most likely tag for the word, and t2...tn are alternate
     tags, in no particular order. */
  /* read through once to get size */
  for (numLexiconEntries=0;fgets(line,sizeof(line),lexicon) != NULL;
       numLexiconEntries+=num_words(line))
    if (not_just_blank(line)) line[strlen(line) - 1] = '\0';

  fseek(lexicon, (long)0 , SEEK_SET);

  /* just need word and most likely tag from lexicon (first tag entry) */
  /* Benjamin Han 100400: originally it's hinted by the # of lines in lexicon
	 file */
  Registry_size_hint(*lexicon_hash,numLexiconEntries);
 
  while(fgets(line,sizeof(line),lexicon) != NULL) {
    if (not_just_blank(line)){
      line[strlen(line) - 1] = '\0';
      sscanf(line,"%s%s",word,tag);
      Registry_add(*lexicon_hash,(char *)mystrdup(word),
		   (char *)mystrdup(tag));
    }
  }

  /* read in lexical rule file */
	  
  while(fgets(line,sizeof(line),lRuleFile) != NULL) {
	if (not_just_blank(line)){
		line[strlen(line) - 1] = '\0';
		Darray_addh(*lRuleArray,mystrdup(line));
		perl_split_ptr = perl_split(line);
		temp_perl_split_ptr = perl_split_ptr;
		if (strcmp(perl_split_ptr[1],"goodright") == 0) {
			tempruleptr = mystrdup(perl_split_ptr[0]);
			Registry_add(*good_right_hash,tempruleptr,(char *)1); }
		else if (strcmp(perl_split_ptr[2],"fgoodright") == 0) {
			tempruleptr = mystrdup(perl_split_ptr[1]);
			Registry_add(*good_right_hash,tempruleptr,(char *)1); }
		else if (strcmp(perl_split_ptr[1],"goodleft") == 0) {
			tempruleptr = mystrdup(perl_split_ptr[0]);
			Registry_add(*good_left_hash,tempruleptr,(char *)1); }
		else if (strcmp(perl_split_ptr[2],"fgoodleft") == 0) {
			tempruleptr = mystrdup(perl_split_ptr[1]);
			Registry_add(*good_left_hash,tempruleptr,(char *)1); }
		free(*perl_split_ptr);
		free(perl_split_ptr);
		}
	}

  /* read in bigram file */
  // Benjamin Han 100400: I store the contents in bigramArray so
  // we don't have to do file IO everytime the start-state-tagger is
  // invoked.

  while(fgets(line,sizeof(line),bigrams) != NULL){
    if(not_just_blank(line)) {
      line[strlen(line) - 1] = '\0';
      atempstr = (char *)malloc(sizeof(char)*(strlen(line)+1));
      strcpy(atempstr,line);
      Darray_addh(*bigramArray,atempstr);
    }
  }

  fseek(lexicon, (long)0 , SEEK_SET);

  /* read in the lexicon for the final-state-tagger */
  Registry_size_hint(*seenTagging,numLexiconEntries);

  // Benjamin Han 100500: MISSING RESTRICT_MOVE section?
  // Answer: Brill used registry WORDS while I use lexicon_hash to replace
  //         his WORDS (see POST::Run) - the only difference is in WORDS 
  //         every value is 1 while in lexicon_hash a values is the first 
  //         tag following the word in the lexicon file.

  while(fgets(line,sizeof(line),lexicon) != NULL) {
    if (not_just_blank(line)) {
      line[strlen(line) - 1] = '\0';
      perl_split_ptr = perl_split(line);
      perl_split_ptr2 = perl_split_ptr;
      ++perl_split_ptr;
      while(*perl_split_ptr != NULL) {
	sprintf(space,"%s %s",*perl_split_ptr2,*perl_split_ptr);
	Registry_add(*seenTagging,mystrdup(space),(char *)1);
	++perl_split_ptr; }
      free(*perl_split_ptr2);
      free(perl_split_ptr2);
    }
  }


  // read in contextual rule
  while(fgets(line, sizeof(line), cRuleFile) != NULL)
    if (not_just_blank(line)) 
      {
	line[strlen(line) - 1] = '\0';
	Darray_addh(*cRuleArray,mystrdup(line));
      }
}


