/*
Java version of Brill's Part-of-Speech Tagger
Copyright (C) 2003-2004, Jimmy Lin

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, please visit 
http://www.gnu.org/copyleft/gpl.html

*/
package edu.mit.csail.brill;

import junit.framework.TestCase;

public class BrillTaggerTest extends TestCase {
    static {
        System.loadLibrary("BTagger");
    }

    private BrillTagger brillTagger;

    protected void setUp() {
        brillTagger = new BrillTagger();
        brillTagger.setup();
    }

    public void testTag1() {
        String test = "John loves Mary";
        Sentence result = brillTagger.tag(test);

        String[][] t = { { "John", "NNP" }, {
                "loves", "VBZ" }, {
                "Mary", "NNP" }
        };

        Sentence gold = new Sentence(test, BrillTagger.tokenize(test), t);

        assertTrue(result.equals(gold));
    }

    public void testTag2() {
        String test = "John loves Mary.";
        Sentence result = brillTagger.tag(test);

        String[][] t = { { "John", "NNP" }, {
                "loves", "VBZ" }, {
                "Mary", "NNP" }, {
                ".", "." }
        };

        Sentence gold = new Sentence(test, BrillTagger.tokenize(test), t);

        assertTrue(result.equals(gold));
    }

    public void testTag3() {
        String test = "John, loves Mary!";
        Sentence result = brillTagger.tag(test);

        String[][] t = { { "John", "NNP" }, {
                ",", "," }, {
                "loves", "VBZ" }, {
                "Mary", "NNP" }, {
                "!", "." }
        };

        Sentence gold = new Sentence(test, BrillTagger.tokenize(test), t);

        assertTrue(result.equals(gold));
    }

    public void testTag4() {
        String test = "\"I love Mary!\" exclaimed John.";
        Sentence result = brillTagger.tag(test);

        String[][] t = { { "\"", "\"" }, {
                "I", "PRP" }, {
                "love", "VBP" }, {
                "Mary", "NNP" }, {
                "!", "." }, {
                "\"", "\"" }, {
                "exclaimed", "VBD" }, {
                "John", "NNP" }, {
                ".", "." }
        };

        Sentence gold = new Sentence(test, BrillTagger.tokenize(test), t);

        assertTrue(result.equals(gold));
    }

    public void testTag5() {
        String test = "``I love Mary,'' said John.";
        Sentence result = brillTagger.tag(test);

        String[][] t = { { "``", "``" }, {
                "I", "PRP" }, {
                "love", "VBP" }, {
                "Mary", "NNP" }, {
                ",", "," }, {
                "''", "''" }, {
                "said", "VBD" }, {
                "John", "NNP" }, {
                ".", "." }
        };

        Sentence gold = new Sentence(test, BrillTagger.tokenize(test), t);

        assertTrue(result.equals(gold));
    }
    
    public void testTag6() {
        // shouldn't break up initials
        String test = "Error with Nicole I. Kidman.";
        Sentence result = brillTagger.tag(test);

        String[][] t = { { "Error", "NNS" }, {
                "with", "IN" }, {
                "Nicole", "NNP" }, {
                "I.", "NNP" }, {
                "Kidman", "NNP" }, {
                ".", "." }
        };

        Sentence gold = new Sentence(test, BrillTagger.tokenize(test), t);

        assertTrue(result.equals(gold));
    }
    

    protected void tearDown() {
        brillTagger.destroy();
    }
}
